"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = void 0;
var fs_1 = require("fs");
var ini_1 = __importDefault(require("ini"));
var common_1 = require("./common");
/**
 * Parses the content of a config file and returns a ConfigModel object.
 * @param content - The content of the config file.
 * @returns The parsed ConfigModel object.
 * @throws Error if the output mode or protocol is invalid.
 */
var parseConfigFile = function (content) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    var config = ini_1.default.parse(content);
    var output = (_a = config.default) === null || _a === void 0 ? void 0 : _a.output;
    if (output && !common_1.VALID_OUTPUT_MODES.includes(output)) {
        throw new Error("Invalid output mode: ".concat(output, ". Valid modes are: ").concat(common_1.VALID_OUTPUT_MODES.join(', ')));
    }
    var protocol = (_b = config.mqtt) === null || _b === void 0 ? void 0 : _b.protocol;
    if (protocol && !common_1.VALID_PROTOCOLS.includes(protocol)) {
        throw new Error("Invalid protocol: ".concat(protocol, ". Valid protocols are: ").concat(common_1.VALID_PROTOCOLS.join(', ')));
    }
    return {
        output: ((_c = config.default) === null || _c === void 0 ? void 0 : _c.output) || common_1.DEFAULT_CONFIG.output,
        mqtt: {
            host: ((_d = config.mqtt) === null || _d === void 0 ? void 0 : _d.host) || common_1.DEFAULT_CONFIG.mqtt.host,
            port: parseInt((_e = config.mqtt) === null || _e === void 0 ? void 0 : _e.port, 10) || common_1.DEFAULT_CONFIG.mqtt.port,
            protocol: ((_f = config.mqtt) === null || _f === void 0 ? void 0 : _f.protocol) || common_1.DEFAULT_CONFIG.mqtt.protocol,
            maxReconnectTimes: parseInt((_g = config.mqtt) === null || _g === void 0 ? void 0 : _g.max_reconnect_times, 10) || common_1.DEFAULT_CONFIG.mqtt.maxReconnectTimes,
            username: ((_h = config.mqtt) === null || _h === void 0 ? void 0 : _h.username) || common_1.DEFAULT_CONFIG.mqtt.username,
            password: ((_j = config.mqtt) === null || _j === void 0 ? void 0 : _j.password) || common_1.DEFAULT_CONFIG.mqtt.password,
        },
    };
};
/**
 * Loads the configuration from a file.
 * If the file exists, it reads the content, parses it, and returns the configuration.
 * If the file doesn't exist or there is an error parsing the content, it returns the default configuration.
 *
 * @returns The loaded configuration.
 */
var loadConfig = function () {
    if ((0, fs_1.existsSync)(common_1.CONFIG_FILE_PATH)) {
        var configFileContent = (0, fs_1.readFileSync)(common_1.CONFIG_FILE_PATH, 'utf-8');
        try {
            return parseConfigFile(configFileContent);
        }
        catch (error) {
            console.error(error.message);
            console.error('Invalid configuration file. Using default configuration.');
        }
    }
    return common_1.DEFAULT_CONFIG;
};
exports.loadConfig = loadConfig;
