"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initConfig = void 0;
var fs_1 = require("fs");
var path_1 = require("path");
var prompts_1 = require("@inquirer/prompts");
var common_1 = require("../configs/common");
/**
 * Generates the content of a configuration INI file based on the provided config object.
 * @param config - The configuration object containing the necessary properties.
 * @returns The generated configuration file content as a string.
 */
var generateConfigContent = function (config) {
    var mqttConfig = "host = ".concat(config.mqtt.host, "\nport = ").concat(config.mqtt.port, "\nprotocol = ").concat(config.mqtt.protocol, "\nmax_reconnect_times = ").concat(config.mqtt.maxReconnectTimes);
    if (config.mqtt.username) {
        mqttConfig += "\nusername = ".concat(config.mqtt.username);
    }
    if (config.mqtt.password) {
        mqttConfig += "\npassword = ".concat(config.mqtt.password);
    }
    return "[default]\noutput = ".concat(config.output, "\n\n[mqtt]\n").concat(mqttConfig);
};
/**
 * Initializes the configuration for MQTTX CLI.
 * Creates or updates the configuration file with the provided values.
 */
function initConfig() {
    return __awaiter(this, void 0, void 0, function () {
        var output, protocol, host, port, maxReconnectTimes, username, passwordAnswer, newConfig;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, (0, prompts_1.select)({
                        message: 'Select MQTTX CLI output mode',
                        choices: [
                            { name: 'Text', value: 'text', description: 'Plain text output' },
                            { name: 'Log', value: 'log', description: 'Log style output' },
                        ],
                        default: common_1.DEFAULT_CONFIG.output,
                    })];
                case 1:
                    output = (_a.sent());
                    return [4 /*yield*/, (0, prompts_1.select)({
                            message: 'Select the default MQTT protocol',
                            choices: [
                                { name: 'MQTT', value: 'mqtt' },
                                { name: 'MQTTS', value: 'mqtts' },
                                { name: 'WS', value: 'ws' },
                                { name: 'WSS', value: 'wss' },
                            ],
                            default: common_1.DEFAULT_CONFIG.mqtt.protocol,
                        })];
                case 2:
                    protocol = (_a.sent());
                    return [4 /*yield*/, (0, prompts_1.input)({
                            message: 'Enter the default MQTT broker host',
                            default: common_1.DEFAULT_CONFIG.mqtt.host,
                        })];
                case 3:
                    host = _a.sent();
                    return [4 /*yield*/, (0, prompts_1.input)({
                            message: 'Enter the default MQTT port',
                            default: common_1.RECOMMENDED_PORTS_BY_PROTOCOL[protocol].toString(),
                            validate: function (input) { return !isNaN(parseInt(input, 10)) || 'Port must be a number'; },
                        })];
                case 4:
                    port = _a.sent();
                    return [4 /*yield*/, (0, prompts_1.input)({
                            message: 'Enter the maximum reconnect times for MQTT connection',
                            default: common_1.DEFAULT_CONFIG.mqtt.maxReconnectTimes.toString(),
                            validate: function (input) { return !isNaN(parseInt(input, 10)) || 'Maximum reconnect times must be a number'; },
                        })];
                case 5:
                    maxReconnectTimes = _a.sent();
                    return [4 /*yield*/, (0, prompts_1.input)({
                            message: 'Enter the default username for MQTT connection authentication',
                            default: common_1.DEFAULT_CONFIG.mqtt.username,
                        })];
                case 6:
                    username = _a.sent();
                    return [4 /*yield*/, (0, prompts_1.password)({
                            message: 'Enter the default password for MQTT connection authentication',
                            mask: true,
                        })];
                case 7:
                    passwordAnswer = _a.sent();
                    newConfig = {
                        output: output,
                        mqtt: {
                            host: host,
                            port: parseInt(port, 10),
                            protocol: protocol,
                            maxReconnectTimes: parseInt(maxReconnectTimes, 10),
                            username: username,
                            password: passwordAnswer,
                        },
                    };
                    try {
                        (0, fs_1.mkdirSync)((0, path_1.join)(common_1.USER_HOME_DIR, '.mqttx-cli'), { recursive: true });
                        (0, fs_1.writeFileSync)(common_1.CONFIG_FILE_PATH, generateConfigContent(newConfig));
                        console.log("Configuration file created/updated at ".concat(common_1.CONFIG_FILE_PATH));
                    }
                    catch (error) {
                        console.error("Unable to create configuration file at ".concat(common_1.CONFIG_FILE_PATH, ". Please check your permissions and try again."));
                    }
                    return [2 /*return*/];
            }
        });
    });
}
exports.initConfig = initConfig;
