"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.benchSub = exports.sub = void 0;
var mqtt = __importStar(require("mqtt"));
var logWrapper_1 = __importStar(require("../utils/logWrapper"));
var parse_1 = require("../utils/parse");
var delay_1 = __importDefault(require("../utils/delay"));
var convertPayload_1 = __importDefault(require("../utils/convertPayload"));
var options_1 = require("../utils/options");
var fileUtils_1 = require("../utils/fileUtils");
var protobuf_1 = require("../utils/protobuf");
var binaryFormats_1 = __importDefault(require("../utils/binaryFormats"));
var Debug = __importStar(require("debug"));
var avro_1 = require("../utils/avro");
var getBenchClientId_1 = __importDefault(require("../utils/getBenchClientId"));
/**
 *
 * Pipeline for processing incoming messages, following two potential steps:
 * 1. Protobuf Deserialization --> Utilized if both protobuf path and message name are defined, otherwise message passes as is.
 * 2. Format Conversion --> Engaged if a format is defined, converting the message accordingly; if not defined, message passes unchanged.
 * Flow:
 *  payload -> Schema [Protobuf, Avro] -> [Format Conversion] -> Processed Message
 * @param payload - The message payload to be processed.
 * @param {SchemaOptions} [schemaOptions] - Options for schema-based encoding
 * @param format - The format of the payload.
 * @returns The processed message as a string or Buffer.
 */
var processReceivedMessage = function (payload, schemaOptions, format) {
    var message = payload;
    var convertMessageFormat = function (msg) {
        if (!format) {
            return msg;
        }
        return (0, convertPayload_1.default)(msg, format, 'decode');
    };
    var deserializeWithSchema = function (msg) {
        if (!schemaOptions)
            return msg;
        switch (schemaOptions.type) {
            case 'protobuf':
                return (0, protobuf_1.deserializeBufferToProtobuf)(payload, schemaOptions.protobufPath, schemaOptions.protobufMessageName, format);
            case 'avro':
                return (0, avro_1.deserializeBufferToAvro)(payload, schemaOptions.avscPath, format ? true : false);
        }
    };
    var pipeline = [deserializeWithSchema, convertMessageFormat];
    message = pipeline.reduce(function (msg, transformer) { return transformer(msg); }, message);
    if (Buffer.isBuffer(message) && format !== 'binary') {
        message = message.toString('utf-8');
    }
    return message;
};
var handleDefaultBinaryFile = function (format, filePath) {
    if (filePath) {
        if ((!format || format !== 'binary') && (0, binaryFormats_1.default)((0, fileUtils_1.getPathExtname)(filePath))) {
            logWrapper_1.default.warn('Please use the --format binary option for handling binary files');
            if (!format) {
                return 'binary';
            }
        }
    }
    return format;
};
var sub = function (options) {
    var loadOptions = options.loadOptions, saveOptions = options.saveOptions;
    loadOptions && (options = (0, options_1.handleLoadOptions)('sub', loadOptions, options));
    saveOptions && (0, options_1.handleSaveOptions)('sub', options);
    options.format = handleDefaultBinaryFile(options.format, options.fileSave || options.fileWrite);
    options.debug && Debug.enable('mqttjs*');
    (0, parse_1.checkTopicExists)(options.topic, 'sub');
    var connOpts = (0, parse_1.parseConnectOptions)(options, 'sub');
    var client = mqtt.connect(connOpts);
    var outputMode = options.outputMode, maximumReconnectTimes = options.maximumReconnectTimes;
    var outputModeClean = outputMode === 'clean';
    var retryTimes = 0;
    !outputModeClean && logWrapper_1.basicLog.connecting(loadOptions, connOpts.hostname, connOpts.port, options.topic.join(', '));
    var subscribeToTopics = function () { return __awaiter(void 0, void 0, void 0, function () {
        var subOptsArray, topic, subscribePromises, results, allSuccessfulSubs, error_1;
        return __generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    if (!outputModeClean)
                        logWrapper_1.basicLog.connected();
                    retryTimes = 0;
                    subOptsArray = (0, parse_1.parseSubscribeOptions)(options);
                    topic = options.topic;
                    if (!outputModeClean) {
                        topic.forEach(function (t) { return logWrapper_1.basicLog.subscribing(t); });
                    }
                    subscribePromises = topic.map(function (t, index) {
                        var subOpts = subOptsArray[index];
                        return new Promise(function (resolve, reject) {
                            client.subscribe(t, subOpts, function (err, result) {
                                if (err) {
                                    if (!outputModeClean)
                                        logWrapper_1.basicLog.error(err);
                                    return reject(err);
                                }
                                var successfulSubs = [];
                                var failedSubs = [];
                                result.forEach(function (sub) {
                                    if (sub.qos > 2) {
                                        failedSubs.push(sub);
                                        if (!outputModeClean)
                                            logWrapper_1.basicLog.subscriptionNegated(sub);
                                    }
                                    else {
                                        successfulSubs.push(sub);
                                        if (!outputModeClean)
                                            logWrapper_1.basicLog.subscribed(sub.topic);
                                    }
                                });
                                resolve({ successfulSubs: successfulSubs, failedSubs: failedSubs });
                            });
                        });
                    });
                    _a.label = 1;
                case 1:
                    _a.trys.push([1, 3, , 4]);
                    return [4 /*yield*/, Promise.all(subscribePromises)];
                case 2:
                    results = _a.sent();
                    allSuccessfulSubs = results.flatMap(function (r) { return r.successfulSubs; });
                    if (allSuccessfulSubs.length === 0) {
                        process.exit(1);
                    }
                    return [3 /*break*/, 4];
                case 3:
                    error_1 = _a.sent();
                    process.exit(1);
                    return [3 /*break*/, 4];
                case 4: return [2 /*return*/];
            }
        });
    }); };
    client.on('connect', subscribeToTopics);
    client.on('message', function (topic, payload, packet) {
        var _a;
        var format = options.format, protobufPath = options.protobufPath, protobufMessageName = options.protobufMessageName, avscPath = options.avscPath, fileSave = options.fileSave, fileWrite = options.fileWrite, delimiter = options.delimiter;
        var schemaOptions = (0, parse_1.parseSchemaOptions)(protobufPath, protobufMessageName, avscPath);
        var msgData = [];
        var receivedMessage = processReceivedMessage(payload, schemaOptions, format);
        var savePath = fileSave ? (0, fileUtils_1.createNextNumberedFileName)(fileSave) : fileWrite;
        if (savePath) {
            fileSave && (0, fileUtils_1.writeFile)(savePath, receivedMessage);
            fileWrite && (0, fileUtils_1.appendFile)(savePath, receivedMessage, delimiter);
        }
        options.verbose && msgData.push({ label: 'mqtt-packet', value: packet });
        msgData.push({ label: 'topic', value: topic });
        msgData.push({ label: 'qos', value: packet.qos });
        packet.retain && msgData.push({ label: 'retain', value: packet.retain });
        if (savePath) {
            var successMessage = fileSave ? 'Saved to file' : 'Appended to file';
            msgData.push({ label: 'payload', value: "".concat(receivedMessage, "\n").concat(successMessage, ": ").concat(savePath) });
        }
        else {
            msgData.push({ label: 'payload', value: receivedMessage });
        }
        if ((_a = packet.properties) === null || _a === void 0 ? void 0 : _a.userProperties) {
            var up_1 = [];
            Object.entries(packet.properties.userProperties).forEach(function (_a) {
                var key = _a[0], value = _a[1];
                if (typeof value === 'string') {
                    up_1.push({ key: key, value: value });
                }
                else {
                    value.forEach(function (v) {
                        up_1.push({ key: key, value: v });
                    });
                }
            });
            msgData.push({ label: 'userProperties', value: up_1 });
        }
        !outputModeClean
            ? (0, logWrapper_1.msgLog)(msgData)
            : console.log(JSON.stringify({ topic: topic, payload: (0, convertPayload_1.default)(payload, format, 'decode'), packet: packet }, null, 2));
    });
    client.on('error', function (err) {
        !outputModeClean && logWrapper_1.basicLog.error(err);
        client.end();
    });
    client.on('reconnect', function () {
        retryTimes += 1;
        if (retryTimes > maximumReconnectTimes) {
            client.end(false, {}, function () {
                !outputModeClean && logWrapper_1.basicLog.reconnectTimesLimit();
            });
        }
        else {
            !outputModeClean && logWrapper_1.basicLog.reconnecting(retryTimes, maximumReconnectTimes);
        }
    });
    client.on('close', function () {
        !outputModeClean && logWrapper_1.basicLog.close();
    });
    client.on('disconnect', function (packet) {
        !outputModeClean && logWrapper_1.basicLog.disconnect(packet);
    });
};
exports.sub = sub;
var benchSub = function (options) { return __awaiter(void 0, void 0, void 0, function () {
    var saveOptions, loadOptions, count, interval, topic, hostname, port, clientId, verbose, maximumReconnectTimes, connOpts, subOptsArray, isNewConnArray, retryTimesArray, interactiveSub, connStart, total, oldTotal, isLogged, connectedCount, subscribedCount, allSuccessfulSubs, failedSubs, i;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                saveOptions = options.saveOptions, loadOptions = options.loadOptions;
                loadOptions && (options = (0, options_1.handleLoadOptions)('benchSub', loadOptions, options));
                saveOptions && (0, options_1.handleSaveOptions)('benchSub', options);
                count = options.count, interval = options.interval, topic = options.topic, hostname = options.hostname, port = options.port, clientId = options.clientId, verbose = options.verbose, maximumReconnectTimes = options.maximumReconnectTimes;
                (0, parse_1.checkTopicExists)(topic, 'benchSub');
                connOpts = (0, parse_1.parseConnectOptions)(options, 'sub');
                subOptsArray = (0, parse_1.parseSubscribeOptions)(options);
                isNewConnArray = Array(count).fill(true);
                retryTimesArray = Array(count).fill(0);
                interactiveSub = new logWrapper_1.Signale({ interactive: true, config: logWrapper_1.singaleConfig });
                logWrapper_1.benchLog.start.sub(loadOptions, count, interval, hostname, port, topic.join(', '));
                connStart = Date.now();
                total = 0;
                oldTotal = 0;
                isLogged = false;
                connectedCount = 0;
                subscribedCount = 0;
                allSuccessfulSubs = [];
                failedSubs = [];
                i = 1;
                _a.label = 1;
            case 1:
                if (!(i <= count)) return [3 /*break*/, 4];
                ;
                (function (i, connOpts) {
                    var opts = __assign({}, connOpts);
                    opts.clientId = (0, getBenchClientId_1.default)(clientId, i, count);
                    var client = mqtt.connect(opts);
                    interactiveSub.await('[%d/%d] - Connecting...', connectedCount, count);
                    client.on('connect', function () { return __awaiter(void 0, void 0, void 0, function () {
                        var connEnd, subscribePromises, logRate, error_2;
                        return __generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    connectedCount += 1;
                                    retryTimesArray[i - 1] = 0;
                                    if (!isNewConnArray[i - 1]) return [3 /*break*/, 5];
                                    interactiveSub.success('[%d/%d] - Connected', connectedCount, count);
                                    if (count === connectedCount) {
                                        connEnd = Date.now();
                                        logWrapper_1.signale.success("Created ".concat(count, " connections in ").concat((connEnd - connStart) / 1000, "s"));
                                    }
                                    subscribePromises = topic.map(function (t, index) {
                                        return new Promise(function (resolve, reject) {
                                            var username = opts.username, clientId = opts.clientId;
                                            var topicName = t.replaceAll('%i', i.toString()).replaceAll('%c', clientId);
                                            username && (topicName = topicName.replaceAll('%u', username));
                                            var subOpts = subOptsArray[index];
                                            client.subscribe(topicName, subOpts, function (err, result) {
                                                if (err) {
                                                    logWrapper_1.default.fail("[".concat(i, "/").concat(count, "] - Client ID: ").concat(opts.clientId, ", ").concat(err));
                                                    return reject(err);
                                                }
                                                result.forEach(function (sub) {
                                                    if (sub.qos > 2) {
                                                        failedSubs.push({
                                                            clientId: opts.clientId,
                                                            subItem: sub,
                                                        });
                                                    }
                                                    else {
                                                        allSuccessfulSubs.push(sub);
                                                    }
                                                    subscribedCount += 1;
                                                });
                                                resolve();
                                            });
                                        });
                                    });
                                    _a.label = 1;
                                case 1:
                                    _a.trys.push([1, 3, , 4]);
                                    return [4 /*yield*/, Promise.all(subscribePromises)];
                                case 2:
                                    _a.sent();
                                    if (connectedCount === count && subscribedCount === count * topic.length && !isLogged) {
                                        if (allSuccessfulSubs.length > 0) {
                                            logWrapper_1.default.success("All connections subscribed");
                                        }
                                        if (failedSubs.length > 0) {
                                            failedSubs.forEach(function (sub) {
                                                logWrapper_1.basicLog.subscriptionNegated(sub.subItem, sub.clientId);
                                            });
                                        }
                                        if (allSuccessfulSubs.length === 0) {
                                            process.exit(1);
                                        }
                                        total = 0;
                                        isLogged = true;
                                        logRate = function () {
                                            var rate = total - oldTotal;
                                            var logMethod = verbose ? logWrapper_1.default.log : interactiveSub.log;
                                            logMethod("Received total: ".concat(total, ", rate: ").concat(rate, "/s"));
                                            oldTotal = total;
                                        };
                                        setInterval(logRate, 1000);
                                    }
                                    return [3 /*break*/, 4];
                                case 3:
                                    error_2 = _a.sent();
                                    process.exit(1);
                                    return [3 /*break*/, 4];
                                case 4: return [3 /*break*/, 6];
                                case 5:
                                    logWrapper_1.benchLog.reconnected(connectedCount, count, opts.clientId);
                                    _a.label = 6;
                                case 6: return [2 /*return*/];
                            }
                        });
                    }); });
                    client.on('message', function () {
                        total += 1;
                    });
                    client.on('error', function (err) {
                        logWrapper_1.benchLog.error(connectedCount, count, opts.clientId, err);
                        client.end();
                    });
                    client.on('reconnect', function () {
                        retryTimesArray[i - 1] += 1;
                        if (retryTimesArray[i - 1] > maximumReconnectTimes) {
                            client.end(false, {}, function () {
                                logWrapper_1.benchLog.reconnectTimesLimit(connectedCount, count, opts.clientId);
                                if (retryTimesArray.findIndex(function (times) { return times <= maximumReconnectTimes; }) === -1) {
                                    process.exit(1);
                                }
                            });
                        }
                        else {
                            logWrapper_1.benchLog.reconnecting(connectedCount, count, opts.clientId);
                            isNewConnArray[i - 1] = false;
                        }
                    });
                    client.on('close', function () {
                        connectedCount > 0 && (connectedCount -= 1);
                        logWrapper_1.benchLog.close(connectedCount, count, opts.clientId);
                    });
                    client.on('disconnect', function (packet) {
                        logWrapper_1.basicLog.disconnect(packet, opts.clientId);
                    });
                })(i, connOpts);
                return [4 /*yield*/, (0, delay_1.default)(interval)];
            case 2:
                _a.sent();
                _a.label = 3;
            case 3:
                i++;
                return [3 /*break*/, 1];
            case 4: return [2 /*return*/];
        }
    });
}); };
exports.benchSub = benchSub;
exports.default = sub;
