"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.description = exports.version = exports.dataFormat = exports.author = exports.name = exports.generator = void 0;
var calculateEnergyConsumption = function (faker, maxPower) {
    var ratedPower = maxPower * 1000;
    var minInstantPower = ratedPower * 0.6;
    var instantPower = faker.number.float({ min: minInstantPower, max: ratedPower, precision: 0.001 });
    var energy = instantPower / 3600;
    return new Number(energy.toFixed(2));
};
var dataCache = {};
var factoryList = [];
var generator = function (faker, options) {
    // Some fields will not change every time data is generated, so store them according to id
    var clientId = options.clientId, count = options.count;
    // Initialize the factory list
    if (!factoryList.length) {
        factoryList = Array.from({ length: count }, function (v, i) {
            return {
                id: "".concat(i).padStart(count.toString().length, '0'),
                name: faker.company.name(),
            };
        });
    }
    if (!dataCache[clientId]) {
        var factory = faker.helpers.arrayElement(factoryList);
        dataCache[clientId] = {
            factory_id: factory.id,
            factory: factory.name,
        };
    }
    var data = __assign(__assign({}, dataCache[clientId]), { values: {
            air_compressor_1: calculateEnergyConsumption(faker, 15),
            air_compressor_2: calculateEnergyConsumption(faker, 20),
            lighting: calculateEnergyConsumption(faker, 5),
            cooling_equipment: calculateEnergyConsumption(faker, 100),
            heating_equipment: calculateEnergyConsumption(faker, 200),
            conveyor: calculateEnergyConsumption(faker, 50),
            coating_equipment: calculateEnergyConsumption(faker, 20),
            inspection_equipment: calculateEnergyConsumption(faker, 10),
            welding_equipment: calculateEnergyConsumption(faker, 20),
            packaging_equipment: calculateEnergyConsumption(faker, 30),
            cutting_equipment: calculateEnergyConsumption(faker, 70),
        }, timestamp: Date.now() });
    return {
        message: JSON.stringify(data),
    };
};
exports.generator = generator;
var name = 'IEM';
exports.name = name;
var author = 'EMQX Team';
exports.author = author;
var dataFormat = 'JSON';
exports.dataFormat = dataFormat;
var version = '0.0.1';
exports.version = version;
var description = 'Simulation to generate Industrial Energy Monitoring data.';
exports.description = description;
