"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.description = exports.version = exports.dataFormat = exports.author = exports.name = exports.generator = void 0;
var dataCache = {};
var generateRoomData = function (faker, roomType) {
    var currentHour = new Date().getHours();
    var isDaytime = currentHour > 6 && currentHour < 20;
    var isSleepingHours = currentHour > 22 || currentHour < 6;
    var isKitchen = roomType === 'kitchen';
    var isBathroom = roomType === 'bathroom';
    var isBedroom = roomType === 'bedroom';
    var baseData = {
        room_type: roomType,
        temperature: faker.number.int({ min: 18, max: 26 }),
        humidity: faker.number.int({ min: 30, max: 50 }),
        lights_on: isDaytime ? faker.datatype.boolean() : !isSleepingHours,
        window_open: isDaytime ? faker.datatype.boolean() : false,
    };
    if (isKitchen) {
        var kitchenData = __assign(__assign({}, baseData), { fridge_temperature: faker.number.int({ min: 2, max: 8 }), oven_on: faker.datatype.boolean() });
        return kitchenData;
    }
    if (isBathroom) {
        var bathroomData = __assign(__assign({}, baseData), { water_tap_running: faker.datatype.boolean(), bath_water_level: faker.number.int({ min: 0, max: 100 }) });
        return bathroomData;
    }
    if (isBedroom) {
        var bedroomData = __assign(__assign({}, baseData), { bed_occupancy: faker.datatype.boolean() });
        return bedroomData;
    }
    return baseData;
};
var generator = function (faker, options) {
    var clientId = options.clientId;
    if (!dataCache[clientId]) {
        dataCache[clientId] = {
            home_id: faker.string.uuid(),
            owner_name: faker.person.fullName(),
            address: faker.location.streetAddress(),
        };
    }
    var roomTypes = ['living room', 'bedroom', 'kitchen', 'bathroom'];
    var data = __assign(__assign({}, dataCache[clientId]), { rooms: roomTypes.map(function (roomType) { return generateRoomData(faker, roomType); }), timestamp: Date.now() });
    return {
        message: JSON.stringify(data),
    };
};
exports.generator = generator;
var name = 'smart_home';
exports.name = name;
var author = 'EMQX Team';
exports.author = author;
var dataFormat = 'JSON';
exports.dataFormat = dataFormat;
var version = '0.0.1';
exports.version = version;
var description = 'Simulation to generate Smart Home data.';
exports.description = description;
