"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeBufferToAvro = exports.serializeAvroToBuffer = exports.getAvroType = exports.clearCache = void 0;
var avro = __importStar(require("avsc"));
var fs = __importStar(require("fs"));
var logWrapper_1 = __importDefault(require("./logWrapper"));
var schemaCache = {};
var clearCache = function () {
    Object.keys(schemaCache).forEach(function (key) { return delete schemaCache[key]; });
};
exports.clearCache = clearCache;
var getAvroType = function (schemaPath) {
    // first search from cache
    if (schemaCache[schemaPath]) {
        return schemaCache[schemaPath];
    }
    try {
        var schemaStr = fs.readFileSync(schemaPath, 'utf-8');
        var parsedSchema = void 0;
        try {
            parsedSchema = JSON.parse(schemaStr);
        }
        catch (err) {
            logWrapper_1.default.fail("Schema not following JSON format: ".concat(err.message));
            process.exit(1);
        }
        var type = avro.Type.forSchema(parsedSchema);
        // cache the parsed schema
        schemaCache[schemaPath] = type;
        return type;
    }
    catch (err) {
        logWrapper_1.default.fail("Unable to load avro schema from ".concat(schemaPath, ": ").concat(err.message));
        process.exit(1);
    }
};
exports.getAvroType = getAvroType;
var serializeAvroToBuffer = function (raw, avroSchemaPath) {
    var type = (0, exports.getAvroType)(avroSchemaPath);
    var rawMessage = raw.toString('utf-8');
    var parsedMessage;
    // Avro requires structured message as input
    try {
        parsedMessage = JSON.parse(rawMessage);
    }
    catch (err) {
        logWrapper_1.default.fail("Invalid JSON input: ".concat(err.message));
        process.exit(1);
    }
    try {
        var serializedMessage = type.toBuffer(parsedMessage);
        return Buffer.from(serializedMessage);
    }
    catch (err) {
        logWrapper_1.default.fail("Unable to serialize message to avro buffer: ".concat(err));
        process.exit(1);
    }
};
exports.serializeAvroToBuffer = serializeAvroToBuffer;
var deserializeBufferToAvro = function (payload, avscSchemaPath, needFormat) {
    var type = (0, exports.getAvroType)(avscSchemaPath);
    try {
        var message = type.fromBuffer(payload);
        if (needFormat) {
            return Buffer.from(JSON.stringify(message));
        }
        return JSON.stringify(message);
    }
    catch (err) {
        logWrapper_1.default.fail("Unable to deserialize avro encoded buffer: ".concat(err.message));
        process.exit(1);
    }
};
exports.deserializeBufferToAvro = deserializeBufferToAvro;
