"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Generate a unique client ID for benchmarking purposes
 * @param clientId - Base client ID string that may contain '%i' placeholder
 * @param index - Index number to replace placeholder or append to client ID
 * @param count - Total count of clients being generated
 * @returns Modified client ID string with index incorporated
 *
 * If clientId contains '%i', replaces all instances with index.
 * If count > 1 and no '%i' placeholder exists, appends '_index' to clientId.
 * Otherwise returns clientId unchanged.
 */
var getBenchClientId = function (clientId, index, count) {
    var hasPlaceholder = clientId.includes('%i');
    var baseClientId = hasPlaceholder ? clientId.replaceAll('%i', index.toString()) : clientId;
    return count > 1 && !hasPlaceholder ? "".concat(baseClientId, "_").concat(index) : baseClientId;
};
exports.default = getBenchClientId;
