"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleLoadOptions = exports.handleSaveOptions = void 0;
var fileUtils_1 = require("./fileUtils");
var logWrapper_1 = __importDefault(require("./logWrapper"));
var defaultPath = "".concat(process.cwd(), "/mqttx-cli-options.json");
var mergeOptions = function (oldConfig, newConfig) { return Object.assign({}, oldConfig, newConfig); };
var removeUselessOptions = function (opts) {
    var saveOptions = opts.saveOptions, loadOptions = opts.loadOptions, rest = __rest(opts, ["saveOptions", "loadOptions"]);
    return rest;
};
/**
 * Filters the options based on their source.
 * @param source - The source of the option values.
 * @param opts - The options to be filtered.
 * @returns The filtered options.
 */
var filterOptions = function (source, opts) {
    return Object.fromEntries(Object.entries(opts).filter(function (_a) {
        var key = _a[0];
        return globalThis.command.getOptionValueSource(key) === source;
    }));
};
/**
 * Validates the options for a specific command type.
 * @param commandType - The type of command.
 * @param filePath - The file path.
 * @param config - The options configuration.
 */
var validateOptions = function (commandType, filePath, config) {
    var data = config[commandType];
    if (!data || typeof data !== 'object' || Object.keys(data).length === 0) {
        logWrapper_1.default.fail("No configuration for ".concat(commandType, " found in ").concat(filePath));
        process.exit(1);
    }
};
/**
 * Handles saving options to a file.
 *
 * @param commandType - The type of command being executed.
 * @param opts - The options to be saved.
 */
var handleSaveOptions = function (commandType, opts) {
    try {
        var filePath = (0, fileUtils_1.processPath)(opts.saveOptions, defaultPath);
        var data = {};
        data[commandType] = removeUselessOptions(opts);
        if ((0, fileUtils_1.fileExists)(filePath)) {
            var config = (0, fileUtils_1.parseYamlOrJson)((0, fileUtils_1.readFile)(filePath).toString(), (0, fileUtils_1.isYaml)(filePath));
            data = mergeOptions(config, data);
        }
        var content = (0, fileUtils_1.stringifyToYamlOrJson)(data, (0, fileUtils_1.isYaml)(filePath));
        (0, fileUtils_1.writeFile)(filePath, content);
        logWrapper_1.default.success("Configurations saved to ".concat(filePath));
    }
    catch (error) {
        var err = error;
        logWrapper_1.default.fail(err.toString());
        process.exit(1);
    }
};
exports.handleSaveOptions = handleSaveOptions;
function handleLoadOptions(commandType, savePath, opts) {
    try {
        var filePath = (0, fileUtils_1.processPath)(savePath, defaultPath);
        if ((0, fileUtils_1.fileExists)(filePath)) {
            var data = (0, fileUtils_1.readFile)(filePath).toString();
            var config = (0, fileUtils_1.parseYamlOrJson)(data, (0, fileUtils_1.isYaml)(filePath));
            validateOptions(commandType, filePath, config);
            return __assign(__assign({}, config[commandType]), filterOptions('cli', opts));
        }
        else {
            logWrapper_1.default.fail("Configuration file ".concat(filePath, " not found"));
            process.exit(1);
        }
    }
    catch (error) {
        var err = error;
        logWrapper_1.default.fail(err.toString());
        process.exit(1);
    }
}
exports.handleLoadOptions = handleLoadOptions;
exports.default = handleSaveOptions;
