"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseSubscribeOptions = exports.parsePublishOptions = exports.parseConnectOptions = exports.parseOutputMode = exports.parseSchemaOptions = exports.parseFormat = exports.parseFileWrite = exports.parseFileSave = exports.parseFileRead = exports.parsePubTopic = exports.checkScenarioExists = exports.checkTopicExists = exports.parseVariadicOfBooleanType = exports.parseQoS = exports.parseKeyValues = exports.parseMQTTVersion = exports.parseProtocol = exports.parseNumber = void 0;
var fs = __importStar(require("fs"));
var logWrapper_1 = __importDefault(require("./logWrapper"));
var generator_1 = require("../utils/generator");
var fileUtils_1 = require("../utils/fileUtils");
var simulate_1 = require("./simulate");
var MQTT_SINGLE_MESSAGE_BYTE_LIMIT = 256 * 1024 * 1024;
var parseNumber = function (value) {
    var parsedValue = Number(value);
    if (isNaN(parsedValue)) {
        logWrapper_1.default.fail("".concat(value, " is not a number."));
        process.exit(1);
    }
    return parsedValue;
};
exports.parseNumber = parseNumber;
var parseProtocol = function (value) {
    if (!['mqtt', 'mqtts', 'ws', 'wss'].includes(value)) {
        logWrapper_1.default.fail('Only mqtt, mqtts, ws and wss are supported.');
        process.exit(1);
    }
    return value;
};
exports.parseProtocol = parseProtocol;
var parseMQTTVersion = function (value) {
    var dict = {
        '3.1': 3,
        '3.1.1': 4,
        '5': 5,
    };
    // Normalize '5.0' to '5'
    var normalizedValue = value === '5.0' ? '5' : value;
    if (!dict[normalizedValue]) {
        logWrapper_1.default.fail('Not a valid MQTT version.');
        process.exit(1);
    }
    return dict[normalizedValue];
};
exports.parseMQTTVersion = parseMQTTVersion;
var parseKeyValues = function (value, previous) {
    var _a, _b;
    var _c = value.split(': '), key = _c[0], val = _c[1];
    if (key && val) {
        if (!previous) {
            return _a = {}, _a[key] = val, _a;
        }
        else {
            if (previous[key]) {
                if (Array.isArray(previous[key])) {
                    ;
                    previous[key].push(val);
                }
                else {
                    previous[key] = [previous[key], val];
                }
                return previous;
            }
            else {
                return __assign(__assign({}, previous), (_b = {}, _b[key] = val, _b));
            }
        }
    }
    else {
        logWrapper_1.default.fail("Invalid key-value pair: \"".concat(value, "\". Expected format is \"key: value\"."));
        process.exit(1);
    }
};
exports.parseKeyValues = parseKeyValues;
var parseQoS = function (value, previous) {
    var parsedValue = Number(value);
    if (isNaN(parsedValue) || parsedValue < 0 || parsedValue > 2) {
        logWrapper_1.default.fail("".concat(value, " is not a valid QoS."));
        process.exit(1);
    }
    else {
        return previous ? __spreadArray(__spreadArray([], previous, true), [parsedValue], false) : [parsedValue];
    }
};
exports.parseQoS = parseQoS;
var parseVariadicOfBooleanType = function (value, previous) {
    if (!['true', 'false'].includes(value)) {
        logWrapper_1.default.fail("".concat(value, " is not a boolean."));
        process.exit(1);
    }
    else {
        var booleanValue = value === 'true';
        return previous ? __spreadArray(__spreadArray([], previous, true), [booleanValue], false) : [booleanValue];
    }
};
exports.parseVariadicOfBooleanType = parseVariadicOfBooleanType;
var checkTopicExists = function (topic, commandType) {
    if (!topic) {
        if (['pub', 'benchPub', 'simulate'].includes(commandType)) {
            console.log("error: required option '-t, --topic <TOPIC>' not specified");
        }
        else if (['sub', 'benchSub'].includes(commandType)) {
            console.log("error: required option '-t, --topic <TOPIC...>' not specified");
        }
        process.exit(1);
    }
};
exports.checkTopicExists = checkTopicExists;
var parsePubTopic = function (value) {
    if (value.includes('+') || value.includes('#')) {
        logWrapper_1.default.fail('You cannot publish the message to a Topic that contains wildcards characters #, +');
        process.exit(1);
    }
    return value;
};
exports.parsePubTopic = parsePubTopic;
var parseFileRead = function (value) {
    var filePath = (0, fileUtils_1.processPath)(value);
    if (!filePath) {
        logWrapper_1.default.fail('A valid file path is required when reading from file.');
        process.exit(1);
    }
    var fileContent = (0, fileUtils_1.readFile)(filePath);
    if (fileContent.length >= MQTT_SINGLE_MESSAGE_BYTE_LIMIT) {
        logWrapper_1.default.fail('File size over 256MB not supported by MQTT.');
        process.exit(1);
    }
    return value;
};
exports.parseFileRead = parseFileRead;
var parseFileSave = function (value) {
    var filePath = (0, fileUtils_1.processPath)(value);
    if (!filePath) {
        logWrapper_1.default.fail('A valid file path is required when saving to file.');
        process.exit(1);
    }
    return filePath;
};
exports.parseFileSave = parseFileSave;
var parseFileWrite = function (value) {
    var filePath = (0, fileUtils_1.processPath)(value);
    if (!filePath) {
        logWrapper_1.default.fail('A valid file path is required when writing to file.');
        process.exit(1);
    }
    return filePath;
};
exports.parseFileWrite = parseFileWrite;
var parseFormat = function (value) {
    if (!['base64', 'json', 'hex', 'cbor', 'binary', 'msgpack'].includes(value)) {
        logWrapper_1.default.fail('Not a valid format type.');
        process.exit(1);
    }
    return value;
};
exports.parseFormat = parseFormat;
var parseSchemaOptions = function (protobufPath, protobufMessageName, avscPath) {
    if (protobufPath && protobufMessageName) {
        return {
            type: 'protobuf',
            protobufPath: protobufPath,
            protobufMessageName: protobufMessageName,
        };
    }
    else if (avscPath) {
        return {
            type: 'avro',
            avscPath: avscPath,
        };
    }
    else {
        return undefined;
    }
};
exports.parseSchemaOptions = parseSchemaOptions;
var parseOutputMode = function (value) {
    if (!['clean', 'default'].includes(value)) {
        logWrapper_1.default.fail('Not a valid output mode.');
        process.exit(1);
    }
    return value;
};
exports.parseOutputMode = parseOutputMode;
var checkScenarioExists = function (name, file) {
    if (!name && !file) {
        console.log("error: required option '-sc, --scenario <SCENARIO>' or '-f, --file <SCENARIO FILE PATH>' not specified");
        process.exit(1);
    }
    if (name) {
        var scenarioList = (0, simulate_1.getLocalScenarioList)();
        if (scenarioList.length === 0) {
            logWrapper_1.default.fail('No local scenario found.');
            process.exit(1);
        }
        if (!scenarioList.includes(name)) {
            logWrapper_1.default.fail("Scenario ".concat(name, " not found in [").concat(scenarioList.join(', '), "]"));
            process.exit(1);
        }
    }
    else if (file) {
        if (!(0, simulate_1.getScenarioFilePath)(file)) {
            logWrapper_1.default.fail("Scenario file ".concat(file, " not found."));
            process.exit(1);
        }
        if (!file.endsWith('.js')) {
            logWrapper_1.default.fail("Scenario file ".concat(file, " is not a JavaScript file."));
            process.exit(1);
        }
    }
};
exports.checkScenarioExists = checkScenarioExists;
var parseWsHeaders = function (wsHeaders, protocol) {
    if (!['ws', 'wss'].includes(protocol)) {
        logWrapper_1.default.fail('WebSocket headers are only supported with WebSocket connections (ws or wss).');
        process.exit(1);
    }
    return {
        headers: wsHeaders,
    };
};
var parseConnectOptions = function (options, commandType) {
    var mqttVersion = options.mqttVersion, hostname = options.hostname, port = options.port, clientId = options.clientId, clean = options.clean, keepalive = options.keepalive, username = options.username, password = options.password, protocol = options.protocol, path = options.path, wsHeaders = options.wsHeaders, key = options.key, cert = options.cert, ca = options.ca, insecure = options.insecure, alpn = options.alpn, reconnectPeriod = options.reconnectPeriod, sessionExpiryInterval = options.sessionExpiryInterval, receiveMaximum = options.receiveMaximum, maximumPacketSize = options.maximumPacketSize, topicAliasMaximum = options.topicAliasMaximum, reqResponseInfo = options.reqResponseInfo, reqProblemInfo = options.reqProblemInfo, willTopic = options.willTopic, willMessage = options.willMessage, willQos = options.willQos, willRetain = options.willRetain, willDelayInterval = options.willDelayInterval, willPayloadFormatIndicator = options.willPayloadFormatIndicator, willMessageExpiryInterval = options.willMessageExpiryInterval, willContentType = options.willContentType, willResponseTopic = options.willResponseTopic, willCorrelationData = options.willCorrelationData, willUserProperties = options.willUserProperties;
    var connectOptions = {
        protocolVersion: mqttVersion,
        hostname: hostname,
        port: port,
        clientId: clientId,
        clean: clean,
        keepalive: keepalive,
        username: username,
        password: password,
        protocol: protocol,
        path: path,
        reconnectPeriod: reconnectPeriod,
    };
    if (key) {
        connectOptions.key = fs.readFileSync(key);
    }
    if (cert) {
        connectOptions.cert = fs.readFileSync(cert);
    }
    if (ca) {
        connectOptions.ca = fs.readFileSync(ca);
    }
    if (insecure) {
        connectOptions.rejectUnauthorized = false;
    }
    if (alpn) {
        connectOptions.ALPNProtocols = alpn;
    }
    if (wsHeaders && protocol) {
        connectOptions.wsOptions = parseWsHeaders(wsHeaders, protocol);
    }
    if (willTopic) {
        var will = {
            topic: willTopic,
            payload: willMessage,
            qos: willQos,
            retain: willRetain,
        };
        connectOptions.will = Object.fromEntries(Object.entries(will).filter(function (_a) {
            var _ = _a[0], v = _a[1];
            return v !== null && v !== undefined;
        }));
        var willProperties = {
            willDelayInterval: willDelayInterval,
            payloadFormatIndicator: willPayloadFormatIndicator,
            messageExpiryInterval: willMessageExpiryInterval,
            contentType: willContentType,
            responseTopic: willResponseTopic,
            correlationData: willCorrelationData,
            userProperties: willUserProperties,
        };
        connectOptions.will &&
            (connectOptions.will.properties = Object.fromEntries(Object.entries(willProperties).filter(function (_a) {
                var _ = _a[0], v = _a[1];
                return v !== null && v !== undefined;
            })));
    }
    var optionsTempWorkAround;
    if (mqttVersion === 3) {
        connectOptions.protocolId = 'MQIsdp';
    }
    else if (mqttVersion === 5) {
        var userProperties = commandType === 'conn' ? options.userProperties : options.connUserProperties;
        var properties = {
            sessionExpiryInterval: sessionExpiryInterval,
            receiveMaximum: receiveMaximum,
            maximumPacketSize: maximumPacketSize,
            topicAliasMaximum: topicAliasMaximum,
            requestResponseInformation: reqResponseInfo,
            requestProblemInformation: reqProblemInfo,
            userProperties: userProperties,
        };
        if (clean === false) {
            if (sessionExpiryInterval !== undefined) {
                properties.sessionExpiryInterval = sessionExpiryInterval;
            }
            else {
                properties.sessionExpiryInterval = parseInt('0xFFFFFFFF', 16);
            }
        }
        connectOptions.properties = Object.fromEntries(Object.entries(properties).filter(function (_a) {
            var _ = _a[0], v = _a[1];
            return v !== null && v !== undefined;
        }));
        // Map options.properties.topicAliasMaximum to options.topicAliasMaximum, as that is where MQTT.js looks for it.
        // TODO: remove after bug fixed in MQTT.js v5.
        optionsTempWorkAround = Object.assign({ topicAliasMaximum: connectOptions.properties ? connectOptions.properties.topicAliasMaximum : undefined }, connectOptions);
    }
    return optionsTempWorkAround || connectOptions;
};
exports.parseConnectOptions = parseConnectOptions;
var parsePublishOptions = function (options) {
    var topic = options.topic, message = options.message, qos = options.qos, retain = options.retain, dup = options.dup, payloadFormatIndicator = options.payloadFormatIndicator, messageExpiryInterval = options.messageExpiryInterval, topicAlias = options.topicAlias, responseTopic = options.responseTopic, correlationData = options.correlationData, userProperties = options.userProperties, subscriptionIdentifier = options.subscriptionIdentifier, contentType = options.contentType, protobufPath = options.protobufPath, protobufMessageName = options.protobufMessageName, avscPath = options.avscPath, format = options.format;
    var publishOptions = {
        qos: qos,
        retain: retain,
        dup: dup,
    };
    var schemaOptions = parseSchemaOptions(protobufPath, protobufMessageName, avscPath);
    if (options.mqttVersion === 5) {
        var properties = {
            payloadFormatIndicator: payloadFormatIndicator,
            messageExpiryInterval: messageExpiryInterval,
            topicAlias: topicAlias,
            responseTopic: responseTopic,
            correlationData: correlationData,
            userProperties: userProperties,
            subscriptionIdentifier: subscriptionIdentifier,
            contentType: contentType,
        };
        publishOptions.properties = Object.fromEntries(Object.entries(properties).filter(function (_a) {
            var _ = _a[0], v = _a[1];
            return v !== null && v !== undefined;
        }));
    }
    return { topic: topic, message: message, schemaOptions: schemaOptions, format: format, opts: publishOptions };
};
exports.parsePublishOptions = parsePublishOptions;
var parseSubscribeOptions = function (options) {
    var mqttVersion = options.mqttVersion, topic = options.topic, qos = options.qos, no_local = options.no_local, retainAsPublished = options.retainAsPublished, retainHandling = options.retainHandling, subscriptionIdentifier = options.subscriptionIdentifier, userProperties = options.userProperties;
    var subOptionsArray = [];
    topic.forEach(function (t, index) {
        var subOptions = {
            qos: (0, generator_1.getSpecialTypesOption)(qos, index, 0),
            nl: (0, generator_1.getSpecialTypesOption)(no_local, index),
            rap: (0, generator_1.getSpecialTypesOption)(retainAsPublished, index),
            rh: (0, generator_1.getSpecialTypesOption)(retainHandling, index),
        };
        if (mqttVersion === 5) {
            var properties = {
                subscriptionIdentifier: (0, generator_1.getSpecialTypesOption)(subscriptionIdentifier, index),
                userProperties: userProperties,
            };
            subOptions.properties = Object.fromEntries(Object.entries(properties).filter(function (_a) {
                var _ = _a[0], v = _a[1];
                return v !== null && v !== undefined;
            }));
        }
        subOptionsArray.push(subOptions);
    });
    return subOptionsArray;
};
exports.parseSubscribeOptions = parseSubscribeOptions;
