"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deserializeBufferToProtobuf = exports.serializeProtobufToBuffer = void 0;
var protobufjs_1 = __importDefault(require("protobufjs"));
var logWrapper_1 = __importDefault(require("./logWrapper"));
var protobufErrors_1 = require("./protobufErrors");
var serializeProtobufToBuffer = function (raw, protobufPath, protobufMessageName) {
    var rawData = raw.toString('utf-8');
    var bufferMessage = Buffer.from(rawData);
    try {
        var root = protobufjs_1.default.loadSync(protobufPath);
        var Message = root.lookupType(protobufMessageName);
        var err = Message.verify(JSON.parse(rawData));
        if (err) {
            logWrapper_1.default.fail("Message serialization error: ".concat(err));
            process.exit(1);
        }
        var data = Message.create(JSON.parse(rawData));
        var serializedMessage = Message.encode(data).finish();
        bufferMessage = Buffer.from(serializedMessage);
    }
    catch (error) {
        logWrapper_1.default.fail("Message serialization error: ".concat(error.message.split('\n')[0]));
        process.exit(1);
    }
    return bufferMessage;
};
exports.serializeProtobufToBuffer = serializeProtobufToBuffer;
var deserializeBufferToProtobuf = function (payload, protobufPath, protobufMessageName, needFormat) {
    try {
        var root = protobufjs_1.default.loadSync(protobufPath);
        var Message = root.lookupType(protobufMessageName);
        var MessageData = Message.decode(payload);
        var err = Message.verify(MessageData);
        if (err) {
            logWrapper_1.default.fail("Message deserialization error: ".concat(err));
            process.exit(1);
        }
        if (needFormat) {
            return Buffer.from(JSON.stringify(MessageData.toJSON()));
        }
        return JSON.stringify(MessageData.toJSON());
    }
    catch (error) {
        var err = (0, protobufErrors_1.transformPBJSError)(error);
        logWrapper_1.default.fail(err.message.split('\n')[0]);
        process.exit(1);
    }
};
exports.deserializeBufferToProtobuf = deserializeBufferToProtobuf;
