"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadSimulator = exports.getScenarioFilePath = exports.getLocalScenarioList = void 0;
var faker_1 = require("@faker-js/faker");
var fs = __importStar(require("fs"));
var path = __importStar(require("path"));
var scenarioFolder = path.join(__dirname, '../scenarios');
var getLocalScenarioList = function () {
    if (!fs.existsSync(scenarioFolder)) {
        return [];
    }
    // Read the files in the scenario folder
    var files = fs
        .readdirSync(scenarioFolder)
        .filter(function (file) { return file.endsWith('.js'); })
        .map(function (file) { return ({
        name: file.replace('.js', ''),
        birthtime: fs.statSync(path.join(scenarioFolder, file)).birthtime.getTime(),
    }); })
        .sort(function (a, b) { return b.birthtime - a.birthtime; })
        .map(function (file) { return file.name; });
    return files;
};
exports.getLocalScenarioList = getLocalScenarioList;
var getScenarioFilePath = function (file) {
    var absoluePath = path.resolve(file);
    if (fs.existsSync(absoluePath)) {
        return absoluePath;
    }
    return '';
};
exports.getScenarioFilePath = getScenarioFilePath;
var loadSimulator = function (name, file) {
    try {
        var filePath = file ? getScenarioFilePath(file) : path.join(scenarioFolder, "".concat(name, ".js"));
        if (!filePath) {
            throw new Error("File not found: ".concat(file || name));
        }
        if (path.extname(filePath) !== '.js') {
            throw new Error("Invalid file type: ".concat(filePath, ". Only .js files are allowed."));
        }
        var simulatorModule_1 = require(filePath);
        if (typeof simulatorModule_1.generator !== 'function') {
            throw new Error('Not a valid simulator module');
        }
        // Rewrite generator to auto inject faker
        var generator = function (options) {
            return simulatorModule_1.generator(faker_1.faker, options);
        };
        return __assign(__assign({}, simulatorModule_1), { generator: generator, name: simulatorModule_1.name || name, file: file, realFilePath: filePath });
    }
    catch (err) {
        throw new Error("Load simulator error: ".concat(err));
    }
};
exports.loadSimulator = loadSimulator;
